"use strict";

import { createHighlighterCore } from '@shikijs/core';
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript';
import React, { useEffect, useMemo, useState } from 'react';
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native';
import { jsx as _jsx } from "react/jsx-runtime";
const FONT_FAMILY = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace'
});
const engine = createJavaScriptRegexEngine({
  forgiving: true,
  simulation: true,
  cache: new Map()
});

// Helper to process the theme's token colors
function processThemeTokenColors(theme) {
  const tokenColorMap = new Map();
  if (!theme.tokenColors) return tokenColorMap;
  theme.tokenColors.forEach(tokenColor => {
    if (!tokenColor.scope || !tokenColor.settings) return;
    const scopes = Array.isArray(tokenColor.scope) ? tokenColor.scope : [tokenColor.scope];
    scopes.forEach(scope => {
      tokenColorMap.set(scope, {
        foreground: tokenColor.settings.foreground,
        fontStyle: tokenColor.settings.fontStyle
      });
    });
  });
  return tokenColorMap;
}
function parseColor(color) {
  if (!color) return undefined;

  // Remove any whitespace and uppercase
  color = color.trim().toUpperCase();

  // Handle rgba/rgb format
  if (color.startsWith('RGB')) {
    return color.toLowerCase();
  }

  // Handle hex colors
  if (color.startsWith('#')) {
    // Convert 3-digit hex to 6-digit
    if (color.length === 4) {
      const r = color[1];
      const g = color[2];
      const b = color[3];
      color = `#${r}${r}${g}${g}${b}${b}`;
    }

    // Handle 8-digit hex (with alpha)
    if (color.length === 9) {
      const r = Number.parseInt(color.slice(1, 3), 16);
      const g = Number.parseInt(color.slice(3, 5), 16);
      const b = Number.parseInt(color.slice(5, 7), 16);
      const a = Number.parseInt(color.slice(7, 9), 16) / 255;
      return `rgba(${r}, ${g}, ${b}, ${a})`;
    }

    // Handle 6-digit hex
    if (color.length === 7) {
      const r = Number.parseInt(color.slice(1, 3), 16);
      const g = Number.parseInt(color.slice(3, 5), 16);
      const b = Number.parseInt(color.slice(5, 7), 16);
      return `rgb(${r}, ${g}, ${b})`;
    }
  }
  return color.toLowerCase();
}
async function createHighlighter({
  langs,
  themes
}) {
  if (!langs?.length) {
    throw new Error('Please provide languages when creating a highlighter.');
  }
  try {
    const highlighter = await createHighlighterCore({
      langs,
      themes,
      engine
    });
    return highlighter;
  } catch (error) {
    console.error('Failed to create highlighter:', error);
    throw error;
  }
}
// Token component moved outside SyntaxHighlighter
const Token = /*#__PURE__*/React.memo(({
  token,
  fontSize,
  textStyle,
  baseStyle,
  tokenColorMap
}) => {
  const tokenStyle = useMemo(() => {
    let defaultForeground;
    let defaultFontStyle;

    // Find matching token color from the theme with proper type safety
    if (token.explanation?.[0]?.scopes) {
      for (const scope of token.explanation[0].scopes) {
        const themeToken = tokenColorMap.get(scope.scopeName);
        if (themeToken) {
          defaultForeground = themeToken.foreground;
          defaultFontStyle = themeToken.fontStyle;
          break;
        }

        // Also check theme matches if available
        const themeMatches = scope.themeMatches;
        if ((themeMatches ?? []).length > 0) {
          // Get the first match safely
          const firstMatch = (themeMatches ?? [])[0];
          if (firstMatch && firstMatch.settings) {
            const {
              settings
            } = firstMatch;
            if (settings.foreground) {
              defaultForeground = settings.foreground;
            }
            if (settings.fontStyle) {
              defaultFontStyle = settings.fontStyle;
            }
          }
        }
      }
    }
    const style = {
      color: parseColor(token.color || defaultForeground),
      fontStyle: token.fontStyle === 2 || defaultFontStyle === 'italic' ? 'italic' : 'normal',
      fontWeight: token.fontStyle === 1 || defaultFontStyle === 'bold' ? '700' : '400',
      textDecorationLine: 'none',
      textAlign: 'left',
      fontSize
    };
    return StyleSheet.flatten([baseStyle, style, textStyle]);
  }, [token, fontSize, baseStyle, textStyle, tokenColorMap]);
  return /*#__PURE__*/_jsx(Text, {
    style: tokenStyle,
    selectable: true,
    children: token.content
  });
});
Token.displayName = 'Token';
const SyntaxHighlighter = ({
  text,
  language,
  languageId,
  theme,
  fontSize = 14,
  scrollViewProps,
  textStyle,
  containerStyle
}) => {
  const [tokens, setTokens] = useState([]);
  const [error, setError] = useState(null);

  // Process theme colors once
  const tokenColorMap = useMemo(() => processThemeTokenColors(theme), [theme]);
  const styles = useMemo(() => {
    const backgroundColor = parseColor(theme.colors?.['editor.background']) || '#2e3440';
    const foregroundColor = parseColor(theme.colors?.['editor.foreground']) || '#d8dee9';
    console.log('[SyntaxHighlighter] Theme colors:', {
      background: backgroundColor,
      foreground: foregroundColor,
      allColors: theme.colors
    });
    return StyleSheet.create({
      scrollView: {
        flex: 1,
        minHeight: 20
      },
      codeContainer: {
        padding: 16,
        minWidth: '100%',
        backgroundColor
      },
      lineContainer: {
        flexDirection: 'row',
        flexWrap: 'wrap'
      }
    });
  }, [theme]);
  const baseStyle = useMemo(() => ({
    fontFamily: FONT_FAMILY,
    color: parseColor(theme.colors?.['editor.foreground']) || '#d8dee9'
  }), [theme]);
  useEffect(() => {
    let mounted = true;
    async function highlight() {
      if (!text) {
        setTokens([]);
        return;
      }
      try {
        console.log('[SyntaxHighlighter] Creating highlighter for language:', languageId);
        const highlighter = await createHighlighter({
          langs: language ? [language] : [],
          themes: [Array.isArray(theme) ? theme[0] : theme]
        });
        console.log('[SyntaxHighlighter] Theme token colors sample:', theme.tokenColors?.slice(0, 5).map(tc => ({
          scope: tc.scope,
          settings: {
            ...tc.settings,
            foreground: tc.settings.foreground,
            fontStyle: tc.settings.fontStyle
          }
        })));
        const result = highlighter.codeToTokens(text, {
          lang: languageId || 'plaintext',
          theme: Array.isArray(theme) ? theme[0] : theme,
          includeExplanation: true
        });
        if (mounted && result.tokens) {
          setTokens(result.tokens);
          setError(null);
        }
      } catch (err) {
        console.error('Syntax highlighting failed:', err);
        if (mounted) {
          setError(err instanceof Error ? err : new Error('Highlighting failed'));
          setTokens([[{
            content: text,
            color: baseStyle.color,
            offset: 0
          }]]);
        }
      }
    }
    highlight();
    return () => {
      mounted = false;
    };
  }, [text, language, languageId, theme, baseStyle.color]);
  if (error) {
    return /*#__PURE__*/_jsx(ScrollView, {
      horizontal: true,
      ...scrollViewProps,
      style: [styles.scrollView, scrollViewProps?.style],
      contentContainerStyle: [styles.codeContainer, containerStyle],
      children: /*#__PURE__*/_jsx(Text, {
        style: [baseStyle, {
          fontSize
        }, textStyle],
        children: text
      })
    });
  }
  return /*#__PURE__*/_jsx(ScrollView, {
    horizontal: true,
    ...scrollViewProps,
    style: [styles.scrollView, scrollViewProps?.style],
    contentContainerStyle: [styles.codeContainer, containerStyle],
    children: /*#__PURE__*/_jsx(View, {
      children: tokens.map((line, lineIndex) => /*#__PURE__*/_jsx(View, {
        style: styles.lineContainer,
        children: line.map((token, tokenIndex) => {
          // Cast token to include the explanation type
          const tokenWithExplanation = token;
          return /*#__PURE__*/_jsx(Token, {
            token: tokenWithExplanation,
            fontSize: fontSize,
            textStyle: textStyle,
            baseStyle: baseStyle,
            tokenColorMap: tokenColorMap
          }, `${lineIndex}-${tokenIndex}`);
        })
      }, lineIndex))
    })
  });
};
export default /*#__PURE__*/React.memo(SyntaxHighlighter);
//# sourceMappingURL=SyntaxHighlighter.js.map